/*:
* @target MZ
* @plugindesc Marrend's Deathblows
* @author Marrend
*
* @help deathblows.js
*
* This plugin provides a mechanic by which a player is prompted to input various buttons on a standard
* attack action. The inspiration of which is Xenogears, and it's Deathblow system.
*
* Players use "A" for a light attack, "S" for a medium attack, and "D" for a heavy attack. While
* Deathblows in Xenogears generally ended with a heavy attack, this plug-in aims for end-user
* customization.
*
*
*
* NOTE TAGS
*
* <maxap: x>
* Sets the maximum AP for the actor to [x].
* Usible by actors or equipment.
* Default value is 3.
*
* <combo skill l: x>
* <combo skill m: x>
* <combo skill h: x>
* Sets the skill IDs for light, medium, and heavy attacks to [x].
* Usible by actors or weapons.
* Default values for all options is defined via plugin parameters, as defined below.
*
* <deathblow: [[string_a, a], [string_b, b], [string_c, c], [etc, etc]]>
* Sets up [string_a] as the combo input for skill id [a].
* Sets up [string_b] as the combo input for skill id [b].
* Sets up [string_c] as the combo input for skill id [c].
* ...and so on.
* Usible by actors or weapons.
* No default values.
*
*
*
* PLUGIN PARAMETERS
*
* These set the default values for the skill IDs of the various attack types, and the skill category ID for Deathblows.
* Initial values for each attack type is 1, and the initial value for the deathblow category is 2.
*
* @param lattack
* @text Light attack
* @desc Default light attack skill ID.
* @default 1
* @type integer
*
* @param mattack
* @text Medium attack
* @desc Default medium attack skill ID.
* @default 1
* @type integer
*
* @param hattack
* @text Heavy attack
* @desc Default heavy attack skill ID.
* @default 1
* @type integer
*
* @param category
* @text Skill category
* @desc Skill category for Deathblows to be skipped over when in combat.
* @default 2
* @type integer
*/

// Input
// Keyboard mapper, copy-pasted from rmmz_core.js
Input.keyMapper = {
    9: "tab", // tab
    13: "ok", // enter
    16: "shift", // shift
    17: "control", // control
    18: "control", // alt
    27: "escape", // escape
    32: "ok", // space
    33: "pageup", // pageup
    34: "pagedown", // pagedown
    37: "left", // left arrow
    38: "up", // up arrow
    39: "right", // right arrow
    40: "down", // down arrow
    45: "escape", // insert
    81: "pageup", // Q
    87: "pagedown", // W
    88: "escape", // X
    90: "ok", // Z
    96: "escape", // numpad 0
    98: "down", // numpad 2
    100: "left", // numpad 4
    102: "right", // numpad 6
    104: "up", // numpad 8
    120: "debug", // F9
	65: "lAttack", // A, light attack
	83: "mAttack", // S, medium attack
	68: "hAttack" // D, heavy attack
};


// Game_Actor
// Sets up Max AP property for actors.
Object.defineProperty(Game_Actor.prototype, "map", {
    get: function() {
        return this.maxAP();
    },
    configurable: true
});

// Function to obtain the value of Max AP.
Game_Actor.prototype.maxAP = function() {
	// Max AP properties from weapons have the highest prorioty.
	if (this.weaponAp()) {
		return this.weaponAp();
	} else if (this.armorAp()) {
	// Max AP properties from armor have the next priority.
		return this.armorAp();
	} else {
		// Max AP properties from actors have the least priority.
		return this.actorAp();
	};
};

// Returns Max AP properties from an equiped weapon, if any.
Game_Actor.prototype.weaponAp = function() {
	notes = [];
	for (let i=0; i<this.weapons().length; i++) {
		notes.push(this.weapons()[i].meta);
	};
	for (let i=0; i<notes.length; i++) {
		// Is there a weapon in this slot?
		if (notes[i]) {
			// Does it have a "maxap" metadata?
			if (notes[i].maxap) {
				// Return it!
				return Number(notes[i].maxap);
			};
		};
	};
	
	// No property found, returns false.
	return false;
};

// Returns Max AP properties from an equiped armor, if any.
Game_Actor.prototype.armorAp = function() {
	notes = [];
	for (let i=0; i<this.armors().length; i++) {
		notes.push(this.armors()[i].meta);
	};
	for (let i=0; i<notes.length; i++) {
		// Is there an armor in this slot?
		if (notes[i]) {
			// Does it have a "maxap" metadata?
			if (notes[i].maxap) {
				// Return it!
				return Number(notes[i].maxap);
			};
		};
	};
	
	// No property found, returns false.
	return false;
};

// Returns Max AP properties from the actor, if any.
Game_Actor.prototype.actorAp = function() {
	notes = this.actor().meta;
	if (notes.maxap) {
		return Number(notes.maxap);
	} else {
		return 3;
	};
};

// Returns an attack skill ID based on [type].
Game_Actor.prototype.attackskill = function(type) {
	if(this.weaponAttack(type)) {
		return this.weaponAttack(type);
	} else {
		return this.actorAttack(type);
	};
};

// Returns a skill ID from a weapon, if any is defined. Variable [type] determines the kind of attack.
Game_Actor.prototype.weaponAttack = function(type) {
	notes = [];
	for (let i=0; i<this.weapons().length; i++) {
		notes.push(this.weapons()[i].meta);
	};
	for (let i=0; i<notes.length; i++) {
		// Is there a weapon in this slot?
		if (notes[i]) {	
			switch (type) {
				case "light": // Light attack
					// Does it have the appropraite metadata?
					if (notes[i]["combo skill l"]) {
						// Return it!
						return Number(notes[i]["combo skill l"]);
					};
				case "medium": // Medium attack
					// Does it have the appropriate metadata?
					if (notes[i]["combo skill m"]) {
						// Return it!
						return Number(notes[i]["combo skill m"]);
					};
				case "heavy": // Heavy attack
					// Does it have the appropriate metadata?
					if (notes[i]["combo skill h"]) {
						// Return it!
						return Number(notes[i]["combo skill h"]);
					};
			};
		};
	};
	// No weapon, or metadata found.
	return false;
};

// Returns a skill ID from an actor, in any is defined. Variable [type] determines the kind of attack.
Game_Actor.prototype.actorAttack = function(type) {
	notes = this.actor().meta;
	switch (type) {
		case "light": // Light attack
			if (notes["combo skill l"]) {
				return Number(notes["combo skill l"]);
			} else {
				return this.defaultAttack("light");
			};
		case "medium": // Medium attack
			if (notes["combo skill m"]) {
				return Number(notes["combo skill m"]);
			} else {
				return this.defaultAttack("medium");
			};
		case "heavy": // Heavy attack
			if (notes["combo skill h"]) {
				return Number(notes["combo skill h"]);
			} else {
				return this.defaultAttack("heavy");
			};
	};
};

// Returns the default skill id for a [type] of attack.
Game_Actor.prototype.defaultAttack = function(type) {
	raw_params = PluginManager.parameters('deathblows');
	switch (type) {
		case "light": // Light attack
			return JSON.parse(raw_params.lattack);
		case "medium": // Medium attack
			return JSON.parse(raw_params.mattack);
		case "heavy": // Heavy attack
			return JSON.parse(raw_params.hattack);
	};
};

// Returns the Deathblow array for the actor.
Game_Actor.prototype.getDeathblow = function() {
	if (this.weaponDeathblow()) {
		return this.weaponDeathblow();
	} else {
		return this.actorDeathblow();
	};
};

// Returns the deathblow array from a weapon.
Game_Actor.prototype.weaponDeathblow = function() {
	notes = [];
	for (let i=0; i<this.weapons().length; i++) {
		notes.push(this.weapons()[i].meta);
	};
	for (let i=0; i<notes.length; i++) {
		// Is there a weapon in this slot?
		if (notes[i]) {
			// Does it have the appropriate metadata?
			if (notes[i].deathblow) {
				// Add it!
				array = [];
				dbarray = JSON.parse(notes[i].deathblow);
				for (let j=0; j<dbarray.length; j++) {
					array.push(dbarray[j]);
				};
				return array;
			}
		};
	};
	// No weapon, or metadata, found. Reutns false.
	return false;
};

// Returns the deathblow array from an actor.
Game_Actor.prototype.actorDeathblow = function() {
	array = [];
	notes = this.actor().meta;
	if (notes.deathblow) {
		dbarray = JSON.parse(notes.deathblow)
		for (let i=0; i<dbarray.length; i++) {
			array.push(dbarray[i]);
		};
	};
	return array;
};

// Returns either the button input combo or skill id of a deathblow.
Game_Actor.prototype.returnDeathblow = function(mode) {
	var combo = [];
	for (let i=0; i<this.getDeathblow().length; i++) {
		var input = this.getDeathblow()[i][0]
		var skillId = this.getDeathblow()[i][1]
		if (this.hasSkill(skillId)) {
			if (mode === "input") {
				combo.push(input);
			} else if (mode === "skillId") {
				combo.push(skillId);
			} else {
				combo.push(this.getDeathblow()[i]);
			};
		};
	};
	return combo;
};

// Number of actions taken is a little different.
Game_Actor.prototype.numActions = function() {
	for (let i=0; i<this._actions.length; i++) {
		if (this._actions[i]._item._dataClass === "skill") {
			action = this._actions[i]._item._itemId
			if (action === this.defaultAttack("light") || action === this.defaultAttack("medium") || action === this.defaultAttack("heavy") || this.returnDeathblow("skillId").includes(action)){
				return 1;
			};
		};
	};
	return this._actions.length;
};

Window_Combo.prototype = Object.create(Window_Base.prototype);
Window_Combo.prototype.constructor = Window_Combo;

// Start-up process.
Window_Combo.prototype.initialize = function(rect) {
	Window_Base.prototype.initialize.call(this, rect);
	this.hide();
	this.clearComboString();
	this.setActor($gameParty.members()[0]);
};

// Gets/switches actor info.
Window_Combo.prototype.setActor = function(actor) {
	this.actor = actor;
	this.ap = actor.map;
};

// Clears combo string.
Window_Combo.prototype.clearComboString = function() {
	this.currentCombo = "";
};

// Display current combo.
Window_Combo.prototype.displayCombo = function() {
	this.drawText("Current Combo:", 0, 0, this.contents.width);
	this.drawText(this.currentCombo, 0, this.lineHeight(), this.contents.width);
	this.drawApGauge(this.actor, 0, this.lineHeight()*2, this.contents.width);
};

// Draws the AP gauge.
Window_Combo.prototype.drawApGauge = function(actor, x, y, width) {
	offset = this.contents.measureTextWidth("AP:");
	rate = (width - offset)* this.ap / actor.map;
	this.contents.fillRect(x+offset, y, width-offset, this.lineHeight(), ColorManager.gaugeBackColor());
	this.contents.gradientFillRect(x+offset, y, rate, this.lineHeight(), "#a06000", "#604000");
	this.drawText("AP:", x, y, width);
	this.drawText(this.ap+"/"+this.actor.map, x, y, width, "right");
};

// Frame update.
Window_Combo.prototype.update = function() {
    Window_Base.prototype.update.call(this);
	this.contents.clear();
    this.displayCombo();
};

// Window_ComboList
// Combo display window.
function Window_ComboList() {
    this.initialize(...arguments);
};

Window_ComboList.prototype = Object.create(Window_Combo.prototype);
Window_ComboList.prototype.constructor = Window_ComboList;

// Start-up process.
Window_ComboList.prototype.initialize = function(rect) {
	Window_Combo.prototype.initialize.call(this, rect);
	this.contents.fontSize = 20;
};

// Gets the deathblow match array from current input.
Window_ComboList.prototype.getMatchArray = function() {
	this.matchArray = [];
	var array = this.actor.returnDeathblow();
	if (this.currentCombo.length === 0) {
		for (let i=0; i<array.length; i++) {
			var input = this.actor.returnDeathblow("input")[i].toUpperCase();
			var skillId = this.actor.returnDeathblow("skillId")[i];
			this.matchArray.push([input, $dataSkills[skillId].name]);
		};
	} else {
		for (let i=0; i<array.length; i++) {
			var input = array[i][0].toUpperCase();
			var match = input.match(this.currentCombo);
			if (match) {
				if (match.index === 0) {
					skillId = this.actor.returnDeathblow("skillId")[i];
					this.matchArray.push([input, $dataSkills[skillId].name]);
				};
			};
		};
	};
};

// List item display.
Window_ComboList.prototype.drawItem = function(index) {
	width = this.contents.width / 2 -this.fittingHeight(0);
	y = Math.floor(index/2);
	if (index%2 === 0) {
		this.drawText(this.matchArray[index][1], 0, this.lineHeight()*y, width);
		this.drawText(this.matchArray[index][0], 0, this.lineHeight()*y, width, "right");
	} else {
		this.drawText(this.matchArray[index][1], this.contents.width/2, this.lineHeight()*y, width);
		this.drawText(this.matchArray[index][0], this.contents.width/2, this.lineHeight()*y, width, "right");
	};
};

// Displays list of possible dearthblows based on current combo.
Window_ComboList.prototype.drawAllItems = function() {
	for (let i=0; i<this.matchArray.length; i++) {
		this.drawItem(i);
	};
};

// Frame update.
Window_ComboList.prototype.update = function() {
    Window_Base.prototype.update.call(this);
	if (this.visible) {
		this.contents.clear();
		this.getMatchArray();
		this.drawAllItems();
	};
};

// Window_ComboHelp
function Window_ComboHelp() {
    this.initialize(...arguments);
};

Window_ComboHelp.prototype = Object.create(Window_Base.prototype);
Window_ComboHelp.prototype.constructor = Window_Combo;

Window_ComboHelp.prototype.initialize = function(rect) {
	Window_Base.prototype.initialize.call(this, rect);
	this.contents.fontSize = 18;
	this.hide();
	this.displayText();
};

Window_ComboHelp.prototype.displayText = function() {
	width = this.contents.width / 3;
	this.drawText("A: (L)ight attack, 1 AP", 0, -8, width);
	this.drawText("S: (M)edium attack, 2 AP", width, -8, width);
	this.drawText("D: (H)eavy attack, 3 AP", width*2, -8, width);
};

// Window_Combo
function Window_Combo() {
    this.initialize(...arguments);
};


// Window_ActorCommand
Window_ActorCommand.prototype.addSkillCommands = function() {
    skillTypes = this._actor.skillTypes();
	raw_params = PluginManager.parameters('deathblows');
	dbStypeId = JSON.parse(raw_params.category);
    for (const stypeId of skillTypes) {
		if (stypeId !== dbStypeId) {
			const name = $dataSystem.skillTypes[stypeId];
			this.addCommand(name, "skill", true, stypeId);
		};
    };
};

// Scene_Battle
// Creates all windows. Aliased function.
scenebattle_createallwindows = Scene_Battle.prototype.createAllWindows
Scene_Battle.prototype.createAllWindows = function() {
	this.createComboWindow();
	this.createComboListWindow();
	this.createComboHelp();
	scenebattle_createallwindows.apply(this);
};

// Creates combo input window.
Scene_Battle.prototype.createComboWindow = function() {
	x = 0;
	y = Graphics.height - this.calcWindowHeight(3, true);
	width = Graphics.width / 3;
	height = this.calcWindowHeight(3, false);
	rect = new Rectangle(x, y, width, height);
	this.comboWindow = new Window_Combo(rect);
	this.addWindow(this.comboWindow);
};

// Creates combo match window.
Scene_Battle.prototype.createComboListWindow = function() {
	x = this.comboWindow.width + this.calcWindowHeight(0);
	y = Graphics.height - this.calcWindowHeight(4) - this.calcWindowHeight(0);
	width = Graphics.boxWidth - x;
	height = this.calcWindowHeight(4);
	rect = new Rectangle(x, y, width, height);
	this.comboList = new Window_ComboList(rect);
	this.addWindow(this.comboList);
};

// Creats combo help window.
Scene_Battle.prototype.createComboHelp = function() {
	y  = Graphics.height - this.calcWindowHeight(6);
	width = Graphics.boxWidth;
	height = this.calcWindowHeight(0.6);
	rect = new Rectangle(0, y, width, height);
	this.comboHelp = new Window_ComboHelp(rect);
	this.addWindow(this.comboHelp);
};


// Processes key input process when combo window is open.
Scene_Battle.prototype.comboProcessing = function() {
	actor = BattleManager.actor();
	win = this.comboWindow;
	list = this.comboList;
	action = new Game_Action(actor, false);
	// Ap is empty. Close the window, there should be a combo string together with regular attacks?
	if (win.ap === 0) {
		this.endCombo(actor);
		this.selectNextCommand();
	};
	// Player inputs a light attack.
	if (Input.isTriggered("lAttack")) {
	//if (win._handlers["lattack"]) {
		if (win.ap >= 1) {
			win.currentCombo += "L";
			list.currentCombo = win.currentCombo;
			win.ap -= 1;
			if (this.matchSkill(actor, win.currentCombo, "deathblow")) {
				this.createAction(actor, action, this.getDeathblow(actor, win.currentCombo));
				this.endCombo(actor);
				this.selectNextCommand();
			} else if (this.matchSkill(actor, win.currentCombo, "light")) {
				this.createAction(actor, action, actor.attackskill("light"));
			};
		};
	};
	// Player inputs a medium attack.
	if (Input.isTriggered("mAttack")) {
	//if (win._handlers["mattack"]) {
		if (win.ap >= 2) {
			win.currentCombo += "M";
			list.currentCombo = win.currentCombo;
			win.ap -= 2;
			if (this.matchSkill(actor, win.currentCombo, "deathblow")) {
				this.createAction(actor, action, this.getDeathblow(actor, win.currentCombo));
				this.endCombo(actor);
				this.selectNextCommand();
			} else if (this.matchSkill(actor, win.currentCombo, "medium")) {
				this.createAction(actor, action, actor.attackskill("medium"));
			};
		};
	};
	// Player inputs a heavy attack.
	if (Input.isTriggered("hAttack")) {
	//if (win._handlers["hattack"]) {
		if (win.ap >= 3) {
			win.currentCombo += "H";
			list.currentCombo = win.currentCombo;
			win.ap -= 3;
			if (this.matchSkill(actor, win.currentCombo, "deathblow")) {
				this.createAction(actor, action, this.getDeathblow(actor, win.currentCombo, true));
				this.endCombo(actor);
				win.hide();
				this.selectNextCommand();
			} else if (this.matchSkill(actor, win.currentCombo, "heavy")) {
				this.createAction(actor, action, actor.attackskill("heavy"));
			};
		};
	};
	// Player presses the cancel button during combo processing.
	if (Input.isTriggered("cancel")) {
	//if (win._handlers["cancel"]) {
		SoundManager.playCancel();
		if (win.currentCombo.length > 0) {
			// A combo has been inputted. Hide the window, continue processing.
			this.endCombo(actor);
			this.selectNextCommand();
		} else {
			// A combo has not been inputted. Hide the window, return to targeting?
			this.endCombo();
			this.onSkillCancel();
		};
	};
	Input.clear();
};

// Ends combo processing.
Scene_Battle.prototype.endCombo = function(actor) {
	if (actor) {
		actor._actions = actor._actions.slice(1);
		actor._actionInputIndex = 0;
	};
	this.comboWindow.hide();
	this.comboWindow.clearComboString();
	this.comboList.hide();
	this.comboList.clearComboString();
	this.comboHelp.hide();
	this._actorCommandWindow.show();
};

// Matches the skill that goes onto the stack from current button input.
Scene_Battle.prototype.matchSkill = function(actor, string, type) {
	action = new Game_Action(actor, false);
	switch (type) {
		case "light": // Light attack
			return string[string.length-1] === "L";
		case "medium": // Medium attack
			return string[string.length-1] === "M";
		case "heavy": // Heavy attack
			return string[string.length-1] === "H";
		case "deathblow": // Deathblow check
			return this.getDeathblow(actor, string);
	};
};

// Verifies that the current combo is a deathblow or returns the skill ID for the deathblow.
Scene_Battle.prototype.getDeathblow = function(actor, string, type) {
	var combo = string.toLowerCase();
	for (let i=0; i<actor.returnDeathblow().length; i++) {
		deathblow = actor.returnDeathblow("input")[i];
		if (combo === deathblow) {
			if (type) {
				return actor.returnDeathblow("skillId")[i];
			} else {
				return true;
			};
		};
	};
};

// Pushes an action with a skill ID.
Scene_Battle.prototype.createAction = function(actor, action, skillId) {
	action.setTarget(this._enemyWindow.enemyIndex());
	action.setSkill(skillId);
	actor._actions.push(action);
};

// Processing after you select an enemy to attack?
Scene_Battle.prototype.onEnemyOk = function() {
    const action = BattleManager.inputtingAction();
    action.setTarget(this._enemyWindow.enemyIndex());
	this.hideSubInputWindows();
	if (this._actorCommandWindow.currentSymbol() === "attack") {
		this.comboWindow.setActor(BattleManager.actor());
		this.comboWindow.clearComboString();
		this.comboWindow.show();
		this.comboList.setActor(BattleManager.actor());
		this.comboList.show();
		this.comboHelp.show();
	} else {
		this.selectNextCommand();
	};
};

// Changes input window context.
Scene_Battle.prototype.changeInputWindow = function() {
    this.hideSubInputWindows();
    if (BattleManager.isInputting()) {
        if (BattleManager.actor()) {
			if (this.comboWindow.visible) {
				this._actorCommandWindow.hide();
				this.comboProcessing();
			} else {
            this.startActorCommandSelection();
			};
        } else {
            this.startPartyCommandSelection();
        };
    } else {
        this.endCommandSelection();
    };
};

// When you cancel an action, clear the action array for the canceled actor first.
Scene_Battle.prototype.commandCancel = function() {
    this.selectPreviousCommand();
	var actor = BattleManager.actor();
	if (actor) {
		if (actor._actions.length > 1) {
			for(let i=0; i<actor._actions.length; i++) {
				actor._actions.pop();
			};
		};
	};
};

// Game_Action
// Sets game action detection of standard attack to include light, medium, and heavy attacks.
Game_Action.prototype.isAttack = function() {
	actor = this.subject();
	if (actor.isActor()) {
		return this.item() === $dataSkills[actor.attackSkillId()] ||
			this.item() === $dataSkills[actor.attackskill("light")] ||
			this.item() === $dataSkills[actor.attackskill("medium")] ||
			this.item() === $dataSkills[actor.attackskill("heavy")];
	} else {
		return this.item() === $dataSkills[actor.attackSkillId()];
	};
};
