/*:
* @target MZ
* @plugindesc Marrend's Party size
* @author Marrend
*
* @help partysize.js
*
* This plugin allows for a party size between 2 and 8, modifying menus and sprite placements to account for it.
*
* PLUGIN PARAMETERS
* Sets the party size. Default value is 4.
* 
* @param partysize
* @text Party size
* @desc Maximum number of members in the battle party. Plugin is designed for values between 2 and 8.
* @default 4
* @type integer
*/

function partysizeParameter() {
	raw_params = PluginManager.parameters('partysize');
    return JSON.parse(raw_params.partysize);
};

// Max party size.
Game_Party.prototype.maxBattleMembers = function() {
    return partysizeParameter();
};

// Adjusts line height when party size is large.
Window_StatusBase.prototype.lineHeight = function() {
	if (partysizeParameter() >= 7) {
		return 26;
	} else if (partysizeParameter() >= 5) {
		return 31;
	} else {
		return 36;
	};
};

// Actors displayed in the party menu.
Window_MenuStatus.prototype.numVisibleRows = function() {
    return partysizeParameter();
};

// Ceases to display state icons when party size becomes large.
Window_MenuStatus.prototype.drawActorSimpleStatus = function(actor, x, y) {
    const lineHeight = this.lineHeight();
    const x2 = x + 180;
    this.drawActorName(actor, x, y);
	switch (partysizeParameter()) {
		case 5:
		case 6:
			this.drawActorLevel(actor, x, y + Math.round(lineHeight*0.7));
			this.drawActorIcons(actor, x, y + lineHeight + ImageManager.iconHeight - this.itemPadding()*2);
			break;
		case 7:
		case 8:
			this.drawActorLevel(actor, x, y + Math.round(lineHeight*0.5));
			this.drawActorIcons(actor, x, y + Math.round(lineHeight*0.5) + ImageManager.iconHeight - this.itemPadding()*2);
			break;
		default:
			this.drawActorLevel(actor, x, y + lineHeight);
			this.drawActorIcons(actor, x, y + lineHeight * 2);
			break;
	};
    this.drawActorClass(actor, x2, y);
    this.placeBasicGauges(actor, x2, y + lineHeight);
};

// Font line hieght size adjustment when party size is large.
// Aliased function;
windowmenustatus_initialize = Window_MenuStatus.prototype.initialize;
Window_MenuStatus.prototype.initialize = function(rect) {
	var params = [rect];
	windowmenustatus_initialize.apply(this, params);
	if (partysizeParameter() >= 5) {
		this.contents.fontSize -= 10;
	} else if (partysizeParameter() >= 5) {
		this.contents.fontSize -= 5;
	};
};

// Actors displayed in battle.
Window_BattleStatus.prototype.maxCols = function() {
	switch (partysizeParameter()) {
		case 5:
		case 6:
			return 3;
		case 7:
		case 8:
			return 4;
		default:
			return partysizeParameter();
	};
};

// Reduces item height when the party is large.
Window_BattleStatus.prototype.itemHeight = function() {
	if (partysizeParameter() >= 5) {
		return this.innerHeight / 2;
	} else {
		return this.innerHeight;
	};
};

// Adjusts background placement on the bottom row when party size is large and odd-numbered.
Window_Selectable.prototype.drawItemBackground = function(index) {
    const rect = this.itemRect(index);
		if ($gameParty.inBattle()) {
			if (partysizeParameter() >= 5 && partysizeParameter() % 2 > 0) {
				if (index >= partysizeParameter() / 2) {
				rect.x += this.itemRectWithPadding(0).width / 2;
			};
		};
	};	
    this.drawBackgroundRect(rect);
};

// Adjusts face image placement on the bottom row when party size is large and odd-numbered.
Window_BattleStatus.prototype.drawItemImage = function(index) {
    const actor = this.actor(index);
    const rect = this.faceRect(index);
	if (partysizeParameter() >= 5 && partysizeParameter() % 2 > 0) {
		if (index >= partysizeParameter() / 2) {
			rect.x += this.itemRectWithPadding(0).width / 2;
		};
	};
    this.drawActorFace(actor, rect.x, rect.y, rect.width, rect.height);
};

// Adjusts text and status placement on the bottom row when party size is large and odd-numbered.
Window_BattleStatus.prototype.drawItemStatus = function(index) {
    const actor = this.actor(index);
    const rect = this.itemRectWithPadding(index);
	if (partysizeParameter() >= 5 && partysizeParameter() % 2 > 0) {
		if (index >= partysizeParameter() / 2) {
			rect.x += this.itemRectWithPadding(0).width / 2;
		};
	};
    const nameX = this.nameX(rect);
    const nameY = this.nameY(rect);
    const stateIconX = this.stateIconX(rect);
    const stateIconY = this.stateIconY(rect);
    const basicGaugesX = this.basicGaugesX(rect);
    const basicGaugesY = this.basicGaugesY(rect);
    this.placeTimeGauge(actor, nameX, nameY);
    this.placeActorName(actor, nameX, nameY);
    this.placeStateIcon(actor, stateIconX, stateIconY);
    this.placeBasicGauges(actor, basicGaugesX, basicGaugesY);
};

// Sets actor home.
Sprite_Actor.prototype.setActorHome = function(index) {
	switch (partysizeParameter()) {
		case 2:
			x = 600 + index * 32;
			y = 280 + 24 + index * 48;
			break;
		case 3:
			x = 600 + index * 32;
			y = 280 + 16 + index * 48;
			break;
		case 4:
			x = 600 + index * 32;
			y = 280 + index * 48;
			break;
		case 5:
			if (index > 2) {
				x = 600 + index * 32;
				y = 280 + 24 + (index-3) * 48;
			} else {
				x = 600 + index * 32;
				y = 280 + 16 + index * 48;
			};
			break;
		case 6:
			if (index > 2) {
				x = 600 + index * 32;
				y = 280 + 16 + (index-3) * 48;
			} else {
				x = 600 + index * 32;
				y = 280 + 16 + index * 48;
			};
			break;
		case 7:
			if (index > 3) {
				x = 600 - 24 + index * 32;
				y = 280 + (index-4) * 48;
			} else {
				x = 600 - 24 + index * 32;
				y = 280 - 16 + index * 48;
			};
			break;
		case 8:
			if (index > 3) {
				x = 600 - 48 + index * 32;
				y = 280 + (index-4) * 48;
			} else {
				x = 600 - 48 + index * 32;
				y = 280 + index * 48;
			};
			break;
	};
	this.setHome(x, y);
};
