/*:
* @target MZ
* @plugindesc Marrend's Stat Slopes
* @author Marrend
*
* @help stat_slope.js
*
* This plugin allows for a linear progression for stats on level-up. The basis of the progression is that
* whatever value a stat might have at level 1, when it reaches max level, it will be a multiple of that
* value. The default expectation/functionality is set so that the stats multiply by 10, and that the
* max level is 100, but can be modified through the plugin parameters.
*
*
*
* NOTE TAGS
*
* <paramCurve: [mhp, mmp, atk, def, mat, mdf, spd, luk]>
* Sets the parameter curve for the actor for all stats. Expected values are floating-point values, such
* as `0.4`, or `1.2`, or what-have-you.
* Usable by actors.
* Default values are 0.7.
*
* <paramCurve: [atk, def, mat, mdf, spd, luk]>
* Sets the parameter curve for six stats, allowing MHP to be derived from the value of ATK/DEF and MMP
* to be derived from the value of MAT/MDF. Expected values are floating-point values, such as `0.4`, or
* `1.2`, or what-have-you.
* Usable by actors.
* No default values.
*
*
*
* PLUGIN PARAMETERS
* expRate
* Experience requirement for level 2. Following levels increment by <>. Input a `0` (or less) to follow the curve indicated by the database for the class.
* 
* maxLevel
* Maximium level obtainible across all classes in the game.
* Default value: 100
*
* statMult
* The amount by which a level 1 stat is multiplied by the time a class reaches `maxLevel`.
* Default value: 10
*
* maxHealth
* The amount that MHP would have at level 1 when the stat rate is `1.0`.
* Default value: 1000
*
* maxMana
* The amount that MMP would have at level 1 when the stat rate is `1.0`.
* Default value: 200
*
* maxOther
* The amount that ATK/DEF/MAT/MDF/SPD/LUK would have at level 1 when the stat rate is `1.0`.
* Default value: 25
*
* @param expRate
* @text Experience rate
* @desc Experience requirement for level 2. Remaining levels based on the D&D 3rd Edition experince curve formula. Input a `0` to follow the curve indiccated by the database for the class.
* @default 1000
* @type integer
*
* @param maxLevel
* @text Max level
* @desc Maximium level obtainible across all classes in the game.
* @default 100
* @type integer
*
* @param statMult
* @text Stat multiplier
* @desc The amount by which a level 1 stat is multiplied by the time a class reaches `maxLevel`.
* @default 10
* @type integer
*
* @param maxHealth
* @text Max health at level 1.
* @desc The amount that MHP would have at level 1 when the stat rate is `1.0`.
* @default 1000
* @type integer
*
* @param maxMana
* @text Max mana at level 1.
* @desc The amount that MMP would have at level 1 when the stat rate is `1.0`.
* @default 200
* @type integer
*
* @param maxOther
* @text Max other stats level 1.
* @desc The amount that ATK/DEF/MAT/MDF/SPD/LUK would have at level 1 when the stat rate is `1.0`.
* @default 25
* @type integer
*/

// Returns the value of a specified plugin parameter.
function statParameter(param) {
	raw_params = PluginManager.parameters('stat_slope');
	switch (param) {
		case "expRate":
			return JSON.parse(raw_params.expRate);
		case "maxLevel":
			return JSON.parse(raw_params.maxLevel);
		case "statMult":
			return JSON.parse(raw_params.statMult);
		case "maxHealth":
			return JSON.parse(raw_params.maxHealth);
		case "maxMana":
			return JSON.parse(raw_params.maxMana);
		case "maxOther":
			return JSON.parse(raw_params.maxOther);
	};
};

// Max Level set to plugin parameter across the board.
Game_BattlerBase.prototype.maxLevel = function() {
	return statParameter("maxLevel");
};

// Experience curve creation.
// Aliased method.
gameactor_expforlevel = Game_Actor.prototype.expForLevel;
Game_Actor.prototype.expForLevel = function(level) {
	if (statParameter("expRate") > 0) {
		return Math.round(level * (level - 1) * statParameter("expRate") / 2.0);
	} else {
		gameactor_expforlevel.apply(this)
	};
};

Game_Actor.prototype.maxLevel = function() {
	return Game_Battler.prototype.maxLevel.apply(this);
};

// Transmutes note tag to usible material.
Game_Actor.prototype.paramArray = function() {
	var notes = $dataActors[this._actorId].meta;
	if (notes.paramCurve) {
		return JSON.parse(notes.paramCurve);
	} else {
		return [0.7, 0.7, 0.7, 0.7, 0.7, 0.7, 0.7, 0.7];
	};
};

// Parameter base for actors.
Game_Actor.prototype.paramBase = function(paramId) {
	return this.paramCurve(paramId);
};

// Generates parameter curve.
Game_Actor.prototype.paramCurve = function(paramId) {
	switch (paramId) {
		case 0:
			// MHP
			if (this.paramArray().length === 6) {
				return Math.round((this.paramBase(2)+this.paramBase(3))*this.level/2);
			} else {
				return this.gradeCurve(paramId, this.paramArray()[0]);
			};
		case 1:
			// MMP
			if (this.paramArray().length === 6) {
				return Math.round((this.paramBase(4)+this.paramBase(5))*this.level/2);
			} else {
				return this.gradeCurve(paramId, this.paramArray()[1]);
			};
		case 2:
			// ATK
			if (this.paramArray().length === 6) {
				return this.gradeCurve(paramId, this.paramArray()[0]);
			} else {
				return this.gradeCurve(paramId, this.paramArray()[2]);
			};
		case 3:
			// DEF
			if (this.paramArray().length === 6) {
				return this.gradeCurve(paramId, this.paramArray()[1]);
			} else {
				return this.gradeCurve(paramId, this.paramArray()[3]);
			};
		case 4:
			// MAT
			if (this.paramArray().length === 6) {
				return this.gradeCurve(paramId, this.paramArray()[2]);
			} else {
				return this.gradeCurve(paramId, this.paramArray()[4]);
			};
		case 5:
			// MDF
			if (this.paramArray().length === 6) {
				return this.gradeCurve(paramId, this.paramArray()[3]);
			} else {
				return this.gradeCurve(paramId, this.paramArray()[5]);
			};
		case 6:
			// SPD
			if (this.paramArray().length === 6) {
				return this.gradeCurve(paramId, this.paramArray()[4]);
			} else {
				return this.gradeCurve(paramId, this.paramArray()[6]);
			};
		case 7:
			// LUK
			if (this.paramArray().length === 6) {
				return this.gradeCurve(paramId, this.paramArray()[5]);
			} else {
				return this.gradeCurve(paramId, this.paramArray()[7]);
			};
	};
};

// Sets a linear parameter curve.
Game_Actor.prototype.gradeCurve = function(paramId, grade) {
	switch (paramId) {
		case 0: //MHP
			return Math.ceil((statParameter("maxHealth") * statParameter("statMult") - statParameter("maxHealth")) * grade / (this.maxLevel() - 1) * (this._level - 1) + (statParameter("maxHealth") * grade));
		case 1: //MMP
			return Math.ceil((statParameter("maxMana") * statParameter("statMult") - statParameter("maxMana")) * grade / (this.maxLevel() - 1) * (this._level - 1) + (statParameter("maxMana") * grade));
		default:
			return Math.ceil((statParameter("maxOther") * statParameter("statMult") - statParameter("maxOther")) * grade / (this.maxLevel() - 1) * (this._level - 1) + (statParameter("maxOther") * grade));
	};
};
